package lab;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

import org.h2.tools.Server;

import lab.data.Game;
import lab.data.Player;
import lab.data.Score;
import lab.data.Score.Difficult;
import lab.storage.JpaConnector;
import lombok.extern.log4j.Log4j2;

/**
 * Class <b>App</b> - extends class Application and it is an entry point of the
 * program
 * 
 * @author Java I
 */
@Log4j2
public class App {

	public static void main(String[] args) {
		log.info("Application lauched");

		Server server = startDBWebServer();

		JpaConnector connector = new JpaConnector();
		for (int i = 0; i < 10; i++) {
			connector.save(Game.generateAny());
		}
		for (int i = 0; i < 10; i++) {
			connector.save(Player.generate());
		}
		List<Player> players = connector.getAll(Player.class);
		List<Game> games = connector.getAll(Game.class);
		for (int i = 0; i < 10; i++) {
			connector.save(Score.generate(players, games));
		}
		connector.getEntityManager().clear();
		connector.getAll(Score.class).forEach(System.out::println);
		connector.getAll(Player.class).forEach(App::printPlayer);
		connector.getAll(Game.class).forEach(App::printGame);
		
		System.out.println("-----------null,null-----------------------------");
		connector.findBy(null, null).forEach(System.out::println);
		System.out.println("-----------    ,null-----------------------------");
		connector.findBy("   ", null).forEach(System.out::println);
		System.out.println("-----------'ea',null-----------------------------");
		connector.findBy("ea", null).forEach(System.out::println);
		System.out.println("-----------'ea',medium-----------------------------");
		connector.findBy("ea", Difficult.MEDIUM).forEach(System.out::println);
		System.out.println("-----------'Java',null-----------------------------");
		connector.findBy("Java", Difficult.MEDIUM).forEach(System.out::println);

		waitForKeyPress();
		stopDBWebServer(server);
	}
	
	private static void printPlayer(Player player) {
		System.out.println(player);
		for (Score score : player.getScores()) {
			System.out.println("  -> " + score);
		}
	}

	private static void printGame(Game game) {
		System.out.println(game);
		for (Score score : game.getScores()) {
			System.out.println("  -> " + score);
		}
	}

	private static Server startDBWebServer() {
		// Start HTTP server for access H2 DB for look inside
		try {
			Server server = Server.createWebServer();
			log.info(server.getURL());
			server.start();
			log.info("DB Web server started!");
			return server;
		} catch (SQLException e) {
			log.error("Cannot create DB web server.", e);
			return null;
		}
	}

	private static void stopDBWebServer(Server server) {
		// Stop HTTP server for access H2 DB
		log.info("Ending DB web server BYE.");
		server.stop();
	}

	private static void waitForKeyPress() {
		log.info("Waitnig for Key press (ENTER)");
		try {
			System.in.read();
		} catch (IOException e) {
			log.error("Cannot read input from keyboard.", e);
		}
	}

}