package lab.storage;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.CriteriaQuery;
import jakarta.persistence.criteria.Predicate;
import jakarta.persistence.criteria.Root;
import lab.data.Game_;
import lab.data.MyEntity;
import lab.data.Score;
import lab.data.Score_;

public class JpaConnector {

	private EntityManagerFactory emf;
	private EntityManager em;

	public JpaConnector() {
		emf = Persistence.createEntityManagerFactory("java2");
		em = emf.createEntityManager();
	}

	public <T extends MyEntity> List<T> getAll(Class<T> clazz) {
		return em.createQuery("select e from " + clazz.getSimpleName() + " e", clazz).getResultList();
	}

	public <T extends MyEntity> T save(T entity) {
		T result;
		em.getTransaction().begin();
		if (entity.getId() == null || entity.getId() == 0) {
			em.persist(entity);
			result = entity;
		} else {
			result = em.merge(entity);
		}
		em.getTransaction().commit();
		return result;
	}

	public void delete(List<? extends MyEntity> entities) {
		em.getTransaction().begin();
		for (MyEntity entity : entities) {
			em.remove(entity);
		}
		em.getTransaction().commit();
	}

	public void stop() {
		em.close();
		emf.close();
	}

	public EntityManager getEntityManager() {
		return em;
	}

	public <T> T find(long id, Class<T> clazz) {
		return em.find(clazz, id);
	}

	public void modifyNoPersistOrMerge(long id, Consumer<Score> modificator) {
		em.getTransaction().begin();
		Score score = em.find(Score.class, id);
		modificator.accept(score);
		em.getTransaction().commit();
	}

	public List<Score> findBy(String partialName, Score.Difficult difficult) {
		CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
		CriteriaQuery<Score> criteriaQuery = criteriaBuilder.createQuery(Score.class);
		Root<Score> root = criteriaQuery.from(Score.class);
		List<Predicate> predicates = new ArrayList<>();
		if (partialName != null && !partialName.isBlank()) {
			predicates.add(
					criteriaBuilder.like(root.get(Score_.game).get(Game_.name), String.format("%%%s%%", partialName)));
		}
		if (difficult != null) {
			predicates.add(criteriaBuilder.equal(root.get(Score_.difficult), difficult));
		}
		if (!predicates.isEmpty()) {
			criteriaQuery.where(predicates.toArray(new Predicate[predicates.size()]));
		}
		return em.createQuery(criteriaQuery).getResultList();
	}

}
